# -*- coding: utf-8 -*-
# Copyright 2018 Alex Marchenko
# Distributed under the terms of the Apache License 2.0
from .aerial import Drone, AerialObject, SinusoidalDrone
from .distance import lat_to_rad, lon_to_rad, earth_radius
